function filename = fcnTestBinaryModel(source, segmentation, opts)

% Load vlfeat and matconvnet tools
start;

% Add path to some useful tools
addpath tools;
addpath (fullfile('tools','Hausdorff'));
%

% -------------------------------------------------------------------------
% Setup model
% -------------------------------------------------------------------------
numClasses = 3;    
% Load trained net
fprintf('Loading %s from %s\n', opts.netName, opts.paths.net);
net = load(fullfile(opts.paths.net, opts.netName));

% setup net as DaG network
net = dagnn.DagNN.loadobj(net.net) ;
net.mode = 'test' ;

% remove layers used at training process 
for name = {'objective', 'accuracy'}
  net.removeLayer(name) ;
end

net.meta.normalization.averageImage = reshape(net.meta.normalization.rgbMean,1,1,3) ;
predVar = net.getVarIndex('prediction') ;
inputVar = 'input' ;
imageNeedsToBeMultiple = true ;

if opts.usegpu
    count = fprintf('Setting GPU ........... wait\n');
    gpuDevice(1);
    net.move('gpu') ;
    fprintf(1, repmat('\b',1,count)); %delete line before
    fprintf('Setting GPU ........... DONE!\n');
end

net.mode = 'test' ;

% -------------------------------------------------------------------------
% Test
% -------------------------------------------------------------------------

% make output folder
if ~exist(opts.outputFolder, 'dir'), mkdir(opts.outputFolder); end
% make output folder for test 
if ~exist(fullfile(opts.outputFolder, 'test'), 'dir')
    mkdir(fullfile(opts.outputFolder, 'test'));
end
% make output folder for computed predictions inside test folder
if ~exist(fullfile(opts.outputFolder, 'test', 'pred'), 'dir')
    mkdir(fullfile(opts.outputFolder, 'test', 'pred'));
end
% make output folder for plots inside test folder
if ~exist(fullfile(opts.outputFolder, 'test', 'plots'), 'dir')
    mkdir(fullfile(opts.outputFolder, 'test', 'plots'));
end
% make output folder for results inside test folder ( source image + manual
% green segmentation line + red prediction line )
if ~exist(fullfile(opts.outputFolder, 'test', 'res'), 'dir')
    mkdir(fullfile(opts.outputFolder, 'test', 'res'));
end
% make output folder for mosaic resume images
if ~exist(fullfile(opts.outputFolder, 'test', 'mosaic'), 'dir')
    mkdir(fullfile(opts.outputFolder, 'test', 'mosaic'));
end
for i = 1:numel(segmentation)
   
  % get data and segmentation image name
  name_data = source(i);
  name_seg = segmentation(i) ;
    
  rgbPath = fullfile(opts.paths.testData, name_data);
  labelsPath = fullfile(opts.paths.testSeg, name_seg);
  
  % Load an image     
  count = fprintf('Computing %s ........... wait\n', rgbPath{1});
  rgb = imread(rgbPath{1}) ;
  
  % Load segmentation  
  anno = imread(labelsPath{1}) ; 
  lb = single(anno) ; 
  lb = mod(lb + 1, 256) ; % 0 = ignore, 1 = bkg
  
  % Subtract the mean (color)
  im = bsxfun(@minus, single(rgb), net.meta.normalization.averageImage) ;
  
  % Some networks requires the image to be a multiple of 32 pixels
  if imageNeedsToBeMultiple
    sz = [size(im,1), size(im,2)] ;
    sz_ = round(sz / 32)*32 ;
    im_ = imresize(im, sz_) ;
  else
    im_ = im ;
  end

  if opts.usegpu
    im_ = gpuArray(im_) ;
  end
  
  % compute scores from net and sample
  net.eval({inputVar, im_}) ;
  scores_ = gather(net.vars(predVar).value) ;
  [~,pred_] = max(scores_,[],3) ;

  if imageNeedsToBeMultiple
    pred = imresize(pred_, sz, 'method', 'nearest') ;
  else
    pred = pred_ ;
  end

  % Accumulate errors
  ok = lb > 0 ;
  
  confusion = accumarray([lb(ok),pred(ok)],1,[numClasses numClasses]) ;
  clear info ;
   
  fprintf(1, repmat('\b',1,count)); %delete line before
  fprintf('Computing %s ........... DONE!\n', rgbPath{1});
  
  [sensitivity, specificity, precision, accuracy, jaccard, dice] =  getResults(confusion);
  contrast = getFSContrast(rgb, anno);
  contrast2 = getFSContrast2(rgb, anno);
  results(i).file = rgbPath{1};
  results(i).tp = confusion(9);
  results(i).fn = confusion(6);
  results(i).fp = confusion(8);
  results(i).tn = confusion(5);
  results(i).ROI = sum(anno(anno==2))/2;
  results(i).nonROI = sum(anno(anno==1));
  results(i).sensitivity = sensitivity;
  results(i).specificity = specificity;
  results(i).precision = precision;
  results(i).accuracy = accuracy;
  results(i).jaccard = jaccard;
  results(i).dice = dice;
  results(i).hausdorff = HausdorffDist(pred, lb);
  results(i).contrast = contrast;
  results(i).contrast2 = contrast2;
  imgresult = overlay_seg_and_pred(rgb,anno,pred-2); 
  [~,name,ext] = fileparts(name_data{1});
  imwrite((pred-2),fullfile(opts.outputFolder, 'test', 'pred', name_data{1}),...
      'png');
  imwrite(imgresult,fullfile(opts.outputFolder, 'test', 'res', [name '_res' ext]),...
      'png');
  
end
% write results to file
filename = fullfile(opts.outputFolder, 'test', 'res.csv');
fileId = fopen(filename, 'w');
header = ['File,tp,fn,fp,tn,size_ROI,size_nonROI,Sensitivity,'...
    'Specificity,Precision,Accuracy,Jaccard,Sorensen-Dice,Hausdorf-distance,'...
    'FS_Contrast_Ratio,ABS_Contrast\n'];

fprintf(fileId, header);

for i = 1:numel(segmentation)
    
    fprintf(fileId, '%s,', results(i).file);
    fprintf(fileId, '%d,%d,%d,%d,%d,%d,',results(i).tp, results(i).fn,...
        results(i).fp, results(i).tn, results(i).ROI, results(i).nonROI );
    
    fprintf(fileId, '%f,%f,%f,%f,%f,%f,%f,%f,%f\n', results(i).sensitivity,...
    results(i).specificity, results(i).precision, results(i).accuracy, results(i).jaccard,...
    results(i).dice, results(i).hausdorff, results(i).contrast, results(i).contrast2);

end
% compute total sum for tp,fn,fp,tn
tp = sum([results.tp]);
fn = sum([results.fn]);
fp = sum([results.fp]);
tn = sum([results.tn]);
ROI = sum([results.ROI]);
nonROI = sum([results.nonROI]);

% compute mean and standard deviation for sensitivity, specificity,
% accuracy, jaccard index, sorensen-dice coefficient and FS contrast

sensitivity = mean([results.sensitivity]);
sensitivity_sd= std([results.sensitivity]);

specificity = nanmean([results.specificity]);
specificity_sd= nanstd([results.specificity]);

precision = nanmean([results.precision]);
precision_sd= nanstd([results.precision]);

accuracy = mean([results.accuracy]);
accuracy_sd= std([results.accuracy]);

jaccard = mean([results.jaccard]);
jaccard_sd= std([results.jaccard]);

dice = mean([results.dice]);
dice_sd= std([results.dice]);

hausdorff = mean([results.hausdorff]);
hausdorff_sd = std([results.hausdorff]);

contrast = mean([results.contrast]);
contrast_sd= std([results.contrast]);

contrast2 = mean([results.contrast2]);
contrast2_sd= std([results.contrast2]);

% Print totals to file
fprintf(fileId, 'Total,');
    fprintf(fileId, '%d,%d,%d,%d,%d,%d,', tp, fn, fp, tn, ROI, nonROI );  
    fprintf(fileId, '%f,%f,%f,%f,%f,%f,%f,%f,%f\n', sensitivity,...
        specificity, precision, accuracy, jaccard, dice, hausdorff,...
        contrast, contrast2);
% Print SD
fprintf(fileId, 'SD,');
fprintf(fileId, '--,--,--,--,--,--,');  
fprintf(fileId, '%f,%f,%f,%f,%f,%f,%f,%f,%f\n', sensitivity_sd,...
        specificity_sd, precision_sd, accuracy_sd, jaccard_sd, dice_sd,...
        hausdorff_sd, contrast_sd, contrast2_sd);
    
fclose(fileId);

% Output mean results on screen

fprintf('Total tp: %d\n', tp);
fprintf('Total fn: %d\n', fn);
fprintf('Total fp: %d\n', fp);
fprintf('Total tn: %d\n', tn);

fprintf( 'Sensitivity: %f, SD: %f\n', sensitivity, sensitivity_sd);
fprintf( 'Specificity: %f, SD: %f\n', specificity,  specificity_sd);
fprintf( 'Precision: %f, SD: %f\n', precision,  precision_sd);
fprintf( 'Accuracy: %f, SD: %f\n', accuracy, accuracy_sd);
fprintf( 'Jaccard index: %f, SD: %f\n', jaccard, jaccard_sd);
fprintf( 'Sorensen-Dice: %f, SD: %f\n', dice, dice_sd);
fprintf( 'Hausdorff-distance: %f, SD: %f\n', hausdorff, hausdorff_sd);
fprintf( 'Mean FS contrast: %f, SD: %f\n', contrast, contrast_sd);
fprintf( 'Mean absolute contrast: %f, SD: %f\n', contrast2, contrast2_sd);

% Some plots

% Absolute contrast difference vs Sensitivity
h1 = figure('rend','painters','pos',[100 100 560 420]);
scatter([results.contrast2], [results.sensitivity]);
xlabel('Absolute contrast difference');
ylabel('Sensitivity');
filename = fullfile(opts.outputFolder, 'test', 'plots', 'ABS_contrast_Sensitivity.png');
frame = getframe(h1);
[img, ~] = frame2im(frame);
img = imresize(img,[681, 916]);
imwrite(img, filename);

%saveas(h1,filename);

% Absolute contrast difference vs Specificity
h2 = figure('rend','painters','pos',[100 100 560 420]);
scatter([results.contrast2], [results.specificity]);
xlabel('Absolute contrast difference');
ylabel('Specificity');
filename = fullfile(opts.outputFolder, 'test', 'plots', 'ABS_contrast_Specificity.png');
frame = getframe(h2);
[img, ~] = frame2im(frame);
img = imresize(img,[681, 916]);
imwrite(img, filename);

% Absolute contrast difference vs Precision
h3 = figure('rend','painters','pos',[100 100 560 420]);
scatter([results.contrast2], [results.specificity]);
xlabel('Absolute contrast difference');
ylabel('Precision');
filename = fullfile(opts.outputFolder, 'test', 'plots', 'ABS_contrast_Precision.png');
frame = getframe(h2);
[img, ~] = frame2im(frame);
img = imresize(img,[681, 916]);
imwrite(img, filename);

% Absolute contrast difference vs Accuracy
h4 = figure('rend','painters','pos',[100 100 560 420]);
scatter([results.contrast2], [results.accuracy]);
xlabel('Absolute contrast difference');
ylabel('Accuracy');
filename = fullfile(opts.outputFolder, 'test', 'plots', 'ABS_contrast_Accuracy.png');
frame = getframe(h3);
[img, ~] = frame2im(frame);
img = imresize(img,[681, 916]);
imwrite(img, filename);

% Absolute contrast difference vs Jaccard index
h5 = figure('rend','painters','pos',[100 100 560 420]);
scatter([results.contrast2], [results.jaccard]);
xlabel('Absolute contrast difference');
ylabel('Jaccard index');
filename = fullfile(opts.outputFolder, 'test', 'plots', 'ABS_contrast_Jaccard.png');
frame = getframe(h4);
[img, ~] = frame2im(frame);
img = imresize(img,[681, 916]);
imwrite(img, filename);

% Absolute contrast difference vs Sorensen-Dice coefficient
h6 = figure('rend','painters','pos',[100 100 560 420]);
scatter([results.contrast2], [results.dice]);
xlabel('Absolute contrast difference');
ylabel('Sorensen Dice coefficient');
filename = fullfile(opts.outputFolder, 'test', 'plots', 'ABS_contrast_Dice.png');
frame = getframe(h5);
[img, ~] = frame2im(frame);
img = imresize(img,[681, 916]);
imwrite(img, filename);

% Absolute contrast difference vs hausdorff distance

% Standby while hausDorff distance not fixed
%{
h7 = figure('rend','painters','pos',[100 100 560 420]);
scatter([results.contrast2], [results.hausdorff]);
xlabel('Absolute contrast difference');
ylabel('hausdorf distance');
filename = fullfile(opts.outputFolder, 'test', 'plots', 'ABS_contrast_Hausdorff.png');
frame = getframe(h6);
[img, ~] = frame2im(frame);
img = imresize(img,[681, 916]);
imwrite(img, filename);
%}

% Mosaic of predictions
path = fullfile(opts.outputFolder, 'test', 'pred');
imgsize = 64;
imgarray = resizeimglist(path, imgsize, 'write2file', false, 'crop', true); 
filename = fullfile(opts.outputFolder, 'test', 'mosaic', 'predictions_mosaic.png');
imwrite(vl_imarray(imgarray), filename);

% Mosaic of results (source images with prediction and manual segmentation
% perimeters )
path = fullfile(opts.outputFolder, 'test', 'res');
imgsize = 64;
imgarray = resizeimglist(path, imgsize, 'write2file', false, 'crop', true);
figure;
h8 = imshow(vl_imarray(imgarray,'Spacing',1));
filename = fullfile(opts.outputFolder, 'test', 'mosaic', 'results_mosaic.png');
imwrite(vl_imarray(imgarray), filename);

% Mosaic of scatter plots
path = fullfile(opts.outputFolder, 'test', 'plots');
imgsize = 512;
imgarray = resizeimglist(path, imgsize, 'write2file', false, 'crop', false);
filename = fullfile(opts.outputFolder, 'test', 'mosaic', 'plots_mosaic.png');
imwrite(vl_imarray(cell2mat(imgarray)), filename);

%{
confusionmat=rot90(confusionmat,2);
imagesc(confusionmat);
colorbar;
drawnow;
%}

%{
OLD VERSION not good if you have a cloud of points
function img = overlay_seg_and_pred2(rgb,seg,pred)

% Get perimeter;

segPerimeter= bwperim(seg);
predPerimeter = bwperim(pred);

% set color for segmentation and prediction
segcolor = [0,255,0];
predcolor = [255,0,0];

R_channel = rgb(:,:,1);
G_channel = rgb(:,:,2);
B_channel = rgb(:,:,3);

R_channel(segPerimeter) = segcolor(1);
R_channel(predPerimeter) = predcolor(1);

G_channel(segPerimeter) = segcolor(2);
G_channel(predPerimeter) = predcolor(2);

B_channel(segPerimeter) = segcolor(3);
B_channel(predPerimeter) = predcolor(3);

img = cat(3, R_channel, G_channel, B_channel);
%}
% Make image by overlapping perimeter of ROI and perimeter of predicted ROI
% over source image

function img = overlay_seg_and_pred(rgb,seg,pred)

% Get perimeter;
sz = size(pred);

% set de line width relative to size
linewidth =floor( min(sz)/25);

segPerim= bwperim(seg-1,8);

predPerim = bwperim(pred,8);

% increase line width
segPerimeter=imdilate(double(segPerim),strel('ball',linewidth,linewidth));
segPerimeter = uint8(segPerimeter);
segPerimeter = segPerimeter - linewidth;
segPerimeter = logical(segPerimeter);

predPerimeter=imdilate(double(predPerim),strel('ball',linewidth,linewidth));
predPerimeter = uint8(predPerimeter);
predPerimeter = predPerimeter - linewidth;
predPerimeter = logical(predPerimeter);

% set color for segmentation and prediction
segcolor = [0,255,0];
predcolor = [255,0,0];

R_channel = rgb(:,:,1);
G_channel = rgb(:,:,2);
B_channel = rgb(:,:,3);

R_channel(segPerimeter) = segcolor(1);
R_channel(predPerimeter) = predcolor(1);

G_channel(segPerimeter) = segcolor(2);
G_channel(predPerimeter) = predcolor(2);

B_channel(segPerimeter) = segcolor(3);
B_channel(predPerimeter) = predcolor(3);

img = cat(3, R_channel, G_channel, B_channel);

    

