function [dataSet, segmentationSet] = getTrainSets(opts)
%GETTRAINSETS  reads and list data and segmentation image folders and save 
% it in k-columns matrix format.
%
% GETTRAINSETS picks-up file name randomly from dataset folders, reshape this 
% list to k-columns array, where k is the k-value or fold number, and saves
% dataSet segmentation and source lists as .mat format.
%  
% Required parameters
%
%   'folds', K: columns of output matrix.
%   'datasetPath', PATH: path to source image set.
%   'segmentationSetPath, PATH: path to segmentations of source image set.
%
% Outputs
%
%   dataSet: Randomized file names of source image set in k-columns format.
%   
%   segmentationSet: Segmentations file names of source image set ordered 
%   as dataSet.
%
%   trainSet.mat: Saved .mat file, includes dataSet and segmentationSet.  

% Load vlfeat and matconvnet tools
start;

% check  data
if ((opts.folds <1) || (opts.folds > 20))
    error('folds value must be in range 1:20');
end


% read directory struct
dataList = dir(opts.paths.sourceData);
segmentationList = dir(opts.paths.sourceSeg);
% remove directory name from list (., ..)
for k = length(dataList):-1:1   
    if dataList(k).isdir
        dataList(k) = [];
        segmentationList(k) = [];
        continue
    end
end

if isempty(dataList)
    error('%s: no files at %s\n', mfilename, opts.paths.sourceData); 
end
if isempty(segmentationList)
    error('%s: no files at %s\n', mfilename, opts.paths.sourceSeg);
end

% 'K-fold mode splits data in n-equal analysis sets. Some data must be 
% discarded to get equal sized sets.

l = length(dataList);
    
if rem(l, opts.folds) ~= 0
    %discard some data because we need equal sized sets
    i = fix(l/opts.folds) * opts.folds;
    for n=l:-1:i+1
        fprintf( 'Ignore %s\n',dataList(n).name);
        dataList(n) = [];
        segmentationList(n) = [];
    end
    % generate index to Randomize list
    randomIndex = randperm(i, i); 
else
    % generate index to Randomize list
    randomIndex = randperm(l, l);
end


% make list from images source and segmentation file names 
dataNameList = {dataList.name};
segmentationNameList = {segmentationList.name};
   

% make n-columns matrix one colum per fold with randomized lists.
columns = opts.folds;
if (opts.folds == 1), columns = 2; end % Special case if folds = 1
    
dataSet = reshape(dataNameList(randomIndex), [], columns);
segmentationSet = reshape(segmentationNameList(randomIndex), [], columns);

end


