function nfo = initprocess(varargin)
%INITPROCESS starts training, testing and/or evaluation process.
%
% info = INITPROCESS() starts training, test or evaluation process.

% Configure all general parameters here. Look at BATCH.M to see some
% execution exemples without a gui or change parameters here and launch the
% GUI by executing initprocess.m

% Load vlfeat and matconvnet tools
start ;

% Debug mode 
opts.debug = false;

% root folder
opts.root = 'root_folder' ;

% general data folder
opts.paths.data = fullfile(opts.root, 'data') ;

% Base Model path before training
opts.paths.model = fullfile(opts.root,'models') ;

% data set folder 
opts.paths.dataset = fullfile(opts.root, 'dataset') ;

% Location of trained net, useful for testing and evaluation mode only.
opts.paths.net = fullfile(opts.root, 'trained_nets') ;

% Location of training set.
opts.paths.source = fullfile(opts.paths.dataset, 'source');

% Location of test set
opts.paths.test = fullfile(opts.paths.dataset, 'test');

% path to evaluation folder, put your images here
opts.paths.eval =  fullfile(opts.paths.dataset, 'eval');

% Get parent folder
parentpath = cd(cd('..'));

% Make general output folder.
opts.outputFolder = fullfile(opts.root,'output');

% Base trained Model 
opts.modelName = 'imagenet-vgg-verydeep-16.mat';

% Name of trained net used at test and evaluation process. 
opts.netName = 'net-epoch-50.mat';

% Select mode, options are 'train'(default), 'test' or 'eval'. 
opts.mode = 'train';

% learning rate used at train mode
opts.learningRate = 0.0000001;

% k value in k-fold validation mode.
opts.folds = 3;

% total training desired iterations. You can select 10-fold mode but run
% only some first n iterations
opts.iterations = opts.folds;

%start fold iteration at desired iter number
opts.iterationStart = 1;

% Epochs number.
opts.epochs = 50;

% enable/disable GPU usage
opts.usegpu = true;

% Multiplicative factor for fc8 layer
opts.fc8factor = 1;

% image size input while training net 
opts.imageSize = [64, 64];

% enable gui
opts.modegui = true;

% enable or disable accuracy and objective error plot while computing
opts.plotStatistics = false;

% store all epochs, save to disk each epoch iteration (remember: 1 epoch
% could spend about 1gb on disk)
opts.storeAllEpochs = false;

% parse input parameters
opts = vl_argparse(opts, varargin) ;

% default dir to train set data and segmentations
opts.paths.sourceData = fullfile(opts.paths.source, 'data') ;
opts.paths.sourceSeg = fullfile( opts.paths.source, 'segmentation') ;

% default dir to test set data and segmentations
opts.paths.testData = fullfile(opts.paths.test, 'data'); 
opts.paths.testSeg = fullfile(opts.paths.test, 'segmentation');

% get mode and gpu selection from ui
global mode;
global gpu;

% if gui input is enabled fire choosedialog and get mode and gpu selection
if opts.modegui
   
    choosedialog();
    switch(mode)
        case 'Training'
            opts.mode = 'train';
        case 'Test'
            opts.mode = 'test';
        case 'Evaluation'
            opts.mode = 'evaluate';
        case 'End'
            error('End execution');
        otherwise
            error('unknown mode %s',mode);
    end
    if gpu == 0, opts.usegpu = false;
    elseif gpu == 1, opts.usegpu = true; end
end

% check input for usegpu parameter
if ~islogical(opts.usegpu)
    if (~strcmpi(opts.usegpu, 'false') && ~strcmpi(opts.usegpu, 'true'))     
        error('unknown option: %s\n', opts.usegpu);
    else
        opts.usegpu = ~strcmpi(opts.usegpu, 'false');
    end
end

% make project folders
if ~exist(opts.root, 'dir'),  mkdir(opts.root); end
if ~exist(opts.paths.model, 'dir'), mkdir(opts.paths.model); end
if ~exist(opts.paths.net, 'dir'), mkdir(opts.paths.net); end
if ~exist(opts.paths.data, 'dir'), mkdir(opts.paths.data); end

% image folders
if ~exist(opts.paths.dataset, 'dir'), mkdir(opts.paths.dataset); end
if ~exist(opts.paths.source, 'dir'), mkdir(opts.paths.source); end
if ~exist(opts.paths.sourceData, 'dir'), mkdir(opts.paths.sourceData); end
if ~exist(opts.paths.sourceSeg, 'dir'), mkdir(opts.paths.sourceSeg); end
if ~exist(opts.paths.test, 'dir'), mkdir(opts.paths.test); end
if ~exist(opts.paths.testData, 'dir'), mkdir(opts.paths.testData); end
if ~exist(opts.paths.testSeg, 'dir'), mkdir(opts.paths.testSeg); end
if ~exist(opts.paths.eval, 'dir'), mkdir(opts.paths.eval); end

switch (opts.mode)
    % train mode 
    case 'train'
        % save parameters to parameters.txt file
        writetable(struct2table(opts), fullfile(fullfile(opts.paths.data,'Main_Settings.txt')));      
        % train
        train(opts);
        
    case 'test'
        testSet = fullfile(opts.paths.data, 'testSet.mat');
        source = listFolder(opts.paths.testData);
        source=source(:);
        segmentation = listFolder(opts.paths.testSeg);
        segmentation = segmentation(:);
        save( testSet, 'source', 'segmentation');
        % check if files exists and size of sets are ok
        checkFiles(source,...
            segmentation,...
            opts.paths.testData,...
            opts.paths.testSeg);   
        % test data with trained net
        results = fcnTestBinaryModel(source, segmentation, opts);
          
    case 'evaluate'
        % select image file to be evaluated
        count = fprintf('Select image\n');
        [opts.filename, opts.filepath, opts.filext] =  uigetfile(...
            fullfile(...
            pwd, opts.paths.eval,'*.jpg;*.png;*.bmp'),...
            'Select image');
        fprintf(1, repmat('\b',1,count)); %delete line before
        fprintf('Image: %s\n', opts.filename);
        
        if opts.filename == 0
            error('no file selected\n'); 
        else
            fcnEval(opts);
        end 
    otherwise
        error('unknown mode %s',opts.mode);
end

%--------------------------------------------------------------------------
% TRAINING
%--------------------------------------------------------------------------
function train(opts)
    
    % each iteration uses 1 column as validation set and the remaining 
    % columns as training set
    
    % load data or make new set
    trainSet = fullfile(opts.paths.data, 'trainSet.mat');
    if exist(trainSet, 'file')
        load(trainSet)
    else            
        [source, segmentation] = getTrainSets(opts);
        save( trainSet, 'source', 'segmentation');             
    end
    % some checks
    checkFiles(source, segmentation,...
        opts.paths.sourceData, opts.paths.sourceSeg); 
    
    if numel(source)==0 || numel(segmentation)==0
        fprintf('%s file not found\n', mfilename);
    end
    
    for i=opts.iterationStart:opts.folds
         % Set dir for each iteration
        iterationDir = fullfile(opts.paths.data, ['iter-' int2str(i)]);
        if ~exist(iterationDir, 'dir'), mkdir(iterationDir); end
        % Save settings for debug
        writetable(struct2table(opts), fullfile(fullfile(iterationDir,'settings.txt')));
        % generate or load local set for this iteration.
        iterationSet = fullfile( iterationDir, 'iterationSet.mat');
        if exist(iterationSet, 'file')
            load(iterationSet);             
        else
                                
            trainData = source;
            trainSeg = segmentation;

            valData = trainData(:,i);
            valSeg = trainSeg(:,i);

            % erase validation column
            trainData(:,i)= []; 
            trainSeg(:,i) = []; 

            train = trainData(:);
            validation = valData(:);
            segTrain = trainSeg(:);
            segVal =  valSeg(:);

            rnd_train= randperm(numel(train));
            rnd_validation = randperm(numel(validation));

            train = train(rnd_train);
            segTrain = segTrain(rnd_train);
            validation = validation(rnd_validation);
            segVal = segVal(rnd_validation);

            save(iterationSet, 'train', 'validation', 'segTrain', 'segVal');
        end
    end
    
    for i=opts.iterationStart:opts.folds
        % Set dir for each iteration
        iterationDir = fullfile(opts.paths.data, ['iter-' int2str(i)]);
        iterationSet = fullfile( iterationDir, 'iterationSet.mat');
        load(iterationSet);     
        epoch = findLastCheckpoint(iterationDir) ;

        % resume interrupted process from previous iteration.
        if epoch >= opts.epochs           
            fprintf('\nIteration(Fold) %d: DONE!\n', i);
            continue
        else               
                 
            % Load imdb if exist or make new one
            imdbPath = fullfile(iterationDir, 'imdb.mat');

            if ~exist(imdbPath, 'file')
                imdb = setup(...
                    'train',train,...
                    'validation', validation, ...
                    'segTrain', segTrain,...
                    'segVal', segVal,...
                    'sourcePath', opts.paths.sourceData,...
                    'segmentationPath', opts.paths.sourceSeg);

                save(imdbPath, '-struct', 'imdb');
            end
       
            % Train net with selected parameters. 
            %reset(gpuDevice(1));

            fcnTrainBinaryModel(...
                'expDir', iterationDir,...
                'dataDir', opts.paths.data,...
                'model', fullfile(opts.paths.model, opts.modelName),...
                'epochs', opts.epochs,...
                'imdbPath', imdbPath,...
                'useGPU', opts.usegpu,...
                'modelType', 'fcn8s',...
                'learningRate', opts.learningRate,...
                'fc8factor', opts.fc8factor,...
                'imageSize', opts.imageSize,...
                'plotStatistics', opts.plotStatistics,...
                'storeAllEpochs', opts.storeAllEpochs);
             replot(iterationDir, 'visible', false);
             fprintf('\nIteration(Fold) %d: DONE!\n', i);       
        end       
    end
%--------------------------------------------------------------------------
% CHECK FILES
%--------------------------------------------------------------------------
    function ok = checkFiles(set1, set2, path1, path2)
        ok = false;
        % some checks
        for i=1:numel(set1)
            filename = fullfile(path1, set1(i));
            assert( exist( filename{1}, 'file' ) == 2, '%s not found.',...
                filename{1});
            filename = fullfile(path2, set2(i));
            assert( exist( filename{1}, 'file' ) == 2, '%s not found.',...
                filename{1});
        end
        assert(numel(set1)==numel(set2),...
            'segmentation and source sets differs');
        ok = true;

%--------------------------------------------------------------------------
%  SELECTION MODE 
%--------------------------------------------------------------------------
function choosedialog
    
    global mode;
    global gpu;
    
    d = dialog('Position',[300 300 250 180],'Name','Select mode');
    movegui(d,'center');
    
    txt1 = uicontrol('Parent',d,...
           'Style','text',...
           'Position',[20 130 210 40],...
           'String','Select execution mode');
       
    popup = uicontrol('Parent',d,...
           'Style','popup',...
           'Position',[75 110 90 25],...
           'String',{'Training'; 'Test'; 'Evaluation'},...
           'Callback',@popup_callback);
       
    btn = uicontrol('Parent',d,...
           'Position',[129 20 70 25],...
           'String','Go!',...
           'Callback','delete(gcf)');
       
    btn1 = uicontrol('Parent',d,...
           'Position',[49 20 70 25],...
           'String', 'Cancel',...
           'Callback',@cancel_callback);
       
    checkbox1 = uicontrol('Parent',d,...
           'Style','checkbox',...
           'String', 'GPU',...
           'Value',1,...
           'Position', [75 70 70 25],...
           'Callback',@checkbox_callback);
      
    mode = 'Training';
    gpu = 1;
       
    % Wait for d to close before running to completion
    uiwait(d);
    
   
       function popup_callback(popup,event)
          global mode;
          idx = popup.Value;
          popup_items = popup.String;
          mode = char(popup_items(idx,:));
          
          
       function cancel_callback(btn1,event)
           global mode;
           delete(gcf);
           mode = 'End';
   
          
       function checkbox_callback(checkbox1,event)
          global gpu;
          gpu = checkbox1.Value;
           
% -------------------------------------------------------------------------
function epoch = findLastCheckpoint(folder)
% -------------------------------------------------------------------------
list = dir(fullfile(folder, 'net-epoch-*.mat')) ;
tokens = regexp({list.name}, 'net-epoch-([\d]+).mat', 'tokens') ;
epoch = cellfun(@(x) sscanf(x{1}{1}, '%d'), tokens) ;
epoch = max([epoch 0]) ;